/*
  Tvister - Live status checker
  Copyright(c) 2011 Ndragomirov[jdavid214@gmail.com]
 
  Dual licensed under the MIT and GPL licenses:
  http://www.opensource.org/licenses/mit-license.php
  http://www.gnu.org/licenses/gpl.html
  
*/

var xml2js = require('xml2js');

var parseError = {
		error: 'parse error'
	};

var apis = {
	justin : {
		api : 'http://api.justin.tv/api/stream/list.json',

		channelTag : 'channel',

		parse : function(data, cb) {
			console.log('parsing justin response', data);
			
			try {
				data = JSON.parse(data);
			} catch(e) {
				cb && cb(null, parseError);
			}
			
			data = data.map(function(ch) {
				return {
					thumbnail : ch.channel.screen_cap_url_medium,
					id : ch.channel.login,
					viewers : ch.channel_count,
					link : ch.channel.channel_url,
				};
			});
			
			cb && cb(null, data);
		}
	},
	own3d : {
		api : 'http://api.own3d.tv/live.php',

		channelTag : 'whatever',

		parse : function(data, cb) {

			var parser = new xml2js.Parser();
			
			parser.on('error', function() {
				cb & cb(null, parseError);
			});

			parser.on('end', function(data) {
				data = data.channel.item;

				data = data.map(function(ch) {
					return {
						thumbnail : ch.thumbnail,
						id : ch.link.replace(/.*\/live\/([0-9]+?)/, '$1'),
						viewers : ch.misc['@'].viewers,
						link : ch.link,
					};
				});
				
				cb && cb(null, data);
			});

			parser.parseString(data);

		}
	},
	regame : {
		api : 'http://www.regame.tv/liveview_xml_multiple.php',

		channelTag : 'stream_ids',

		parse : function(data, cb) {

			var parser = new xml2js.Parser();
			
			parser.on('error', function() {
				cb & cb(null, parseError);
			});
			
			parser.on('end', function(data) {

				data = data.stream;
				
				if ( !Array.isArray(data) ) {
					data = [ data ];
				};

				data = data.filter(function(ch) {
					return ch.online === 'true';
				}).map(function(ch) {
					return {
						thumbnail : ch.thumbnail,
						id : ch['@'].id,
						viewers : ch.viewer,
						link : ch.streamlink,
					};
				});

				cb && cb(null, data);
			});

			parser.parseString(data);
		}
	},
	
	livestream : {
		api : 'http://channel.api.livestream.com/1.0/channelsinfo',
		
		thumb: 'http://thumbnail.api.livestream.com/thumbnail?name=CID',
		
		channelTag : 'channel',
		
		parse : function(data, cb) {
			
			
			
			var parser = new xml2js.Parser(),
				thumb = this.thumb;
			
			parser.on('error', function() {
				cb & cb(null, parseError);
			});
			
			parser.on('end', function(data) {
		
				data = data['ls:channel'];

				if ( !Array.isArray(data) ) {
					data = [ data ];
				};

				data = data.filter(function(ch) {
					return ch['ls:isLive'] === 'true';
				}).map(function(ch) {
					return {
						thumbnail : thumb.replace('CID', ch['ls:shortName']),
						id : ch['ls:shortName'],
						viewers : ch['ls:currentViewerCount'],
						link : ch['ls:link'],
					};
				});
				cb && cb(null, data);
			});

			parser.parseString(data);
		}
	},
	/*	twitch : {
	api : 'http://api.justin.tv/api/stream/list.json',
},

xfire : {
	api : 'http://channel.api.livestream.com/1.0/channelsinfo',
},*/
};


module.exports = apis;