/*
  Tvister - live status checker for JustinTV, Own3d, Regame, etc.
  Copyright(c) 2011 Ndragomirov[jdavid214@gmail.com]
  
  Dual licensed under the MIT and GPL licenses:
  http://www.opensource.org/licenses/mit-license.php
  http://www.gnu.org/licenses/gpl.html
  
  Basic example:
  
  tvister
  		.sites({
 			justin: ['justinID1', 'justinID2', 'justinID3' ],
 			own3d:  ['2451', '1234', '422']
  		})
  		.send(function(res) {
  			console.log(res);
		});
*/

var agent   = require('./agent'), 
	APIS    = require('./apis'), 
	async   = require('async'), 
	util    = require('util'),
	isArray = Array.isArray;


var tvister = (function() {

	var supportedSites = Object.keys(APIS),
		channelsContainer = {},
		responseContainer = {},
		parsedDataContainer = {},
		globalcb;
	
	function getData(callback) {
		async.parallel(responseContainer, callback);
	};

	function parseData(err, res) {
		for ( var site in res ) {	
			if ( res[site].error ) continue;
			parsedDataContainer[site] = parserHander( res, site );
		};
		
		async.parallel(parsedDataContainer, function(err, res){
			globalcb(__(res));
		});
	};
	
	function isValidChannel( channel ) {
		return ( typeof channel == 'string' );
	};
	
	function __(data) {
		if ( data.own3d ) {
			data.own3d = data.own3d.filter(function(ch) {
				return channelsContainer.own3d.indexOf(ch.id) >= 0 ;
			});
		};
		console.log('\n\n**** PARSE COMPLETE: ****\n', data);
		return data;
	}
	
	function isSiteSupported( site ) {
		return supportedSites.indexOf( site.toLowerCase() ) >= 0;
	};
	
	function parserHander(response, site) {
		return function(callback) {
			APIS[site].parse( response[site], callback );
		};
	};
	
	function requestHandler(query, site) {
		return function(callback) {
			new agent().options(query).send(APIS[site].api, callback);
		};
	};

	return {

		// setup fetch params
		sites : function( sites ) {
			var site,
				channels;
			
			for ( site in sites) {
				
				if ( isSiteSupported(site) ) {
					channels = sites[site];
					channels = ( isArray(channels) ? channels : [] ).filter(isValidChannel);
					
					if ( channels.length ) {
						channelsContainer[site] = channels;
					};
					
				} else {
					console.log('\nThis site :', site, ' is not supported');
				}
				
			};

			return this;
		},

		timeout : function(value) {
			timeout = value;

			return this;
		},

		send : function(cb) {
			var channels;
			
			globalcb = cb;

			for ( var site in channelsContainer) {
				channels = channelsContainer[site];
				console.log('channels', channels);
				var query = {};
				query[APIS[site].channelTag] = channels.join(',');
				responseContainer[site] = requestHandler(query, site);

			}
			
			getData(parseData);
			return this;
		}
	};
})();

module.exports.version = '0.0.1a';

module.exports.create = function() {
	return tvister;
};